You are viewing a plain text version of this content. The canonical link for it is here.
Posted to commits@poi.apache.org by fa...@apache.org on 2021/10/19 14:01:40 UTC

svn commit: r1894378 - in /poi/trunk/poi/src: main/java/org/apache/poi/ss/formula/atp/ main/java/org/apache/poi/ss/formula/eval/ main/java/org/apache/poi/ss/formula/functions/ test/java/org/apache/poi/ss/formula/functions/

Author: fanningpj
Date: Tue Oct 19 14:01:40 2021
New Revision: 1894378

URL: http://svn.apache.org/viewvc?rev=1894378&view=rev
Log:
[bug-64258] support TDIST function

Added:
    poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/functions/TDist.java
      - copied, changed from r1894356, poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/functions/Delta.java
    poi/trunk/poi/src/test/java/org/apache/poi/ss/formula/functions/TestTDist.java
      - copied, changed from r1894356, poi/trunk/poi/src/test/java/org/apache/poi/ss/formula/functions/TestDelta.java
Modified:
    poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/atp/AnalysisToolPak.java
    poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/eval/FunctionEval.java

Modified: poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/atp/AnalysisToolPak.java
URL: http://svn.apache.org/viewvc/poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/atp/AnalysisToolPak.java?rev=1894378&r1=1894377&r2=1894378&view=diff
==============================================================================
--- poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/atp/AnalysisToolPak.java (original)
+++ poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/atp/AnalysisToolPak.java Tue Oct 19 14:01:40 2021
@@ -173,6 +173,7 @@ public final class AnalysisToolPak imple
         r(m, "TBILLEQ", null);
         r(m, "TBILLPRICE", null);
         r(m, "TBILLYIELD", null);
+        //r(m, "TDIST.AR", TDist.instance);
         r(m, "TEXTJOIN", TextJoinFunction.instance);
         r(m, "WEEKNUM", WeekNum.instance);
         r(m, "WORKDAY", WorkdayFunction.instance);

Modified: poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/eval/FunctionEval.java
URL: http://svn.apache.org/viewvc/poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/eval/FunctionEval.java?rev=1894378&r1=1894377&r2=1894378&view=diff
==============================================================================
--- poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/eval/FunctionEval.java (original)
+++ poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/eval/FunctionEval.java Tue Oct 19 14:01:40 2021
@@ -281,7 +281,7 @@ public final class FunctionEval {
         retval[298] = NumericFunction.ODD;
         // 299: PERMUT
         retval[300] = NumericFunction.POISSON;
-        // 301: TDIST
+        retval[301] = TDist.instance;
         // 302: WEIBULL
         retval[303] = new Sumxmy2();
         retval[304] = new Sumx2my2();

Copied: poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/functions/TDist.java (from r1894356, poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/functions/Delta.java)
URL: http://svn.apache.org/viewvc/poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/functions/TDist.java?p2=poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/functions/TDist.java&p1=poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/functions/Delta.java&r1=1894356&r2=1894378&rev=1894378&view=diff
==============================================================================
--- poi/trunk/poi/src/test/java/org/apache/poi/ss/formula/functions/TestDelta.java (original)
+++ poi/trunk/poi/src/test/java/org/apache/poi/ss/formula/functions/TestTDist.java Tue Oct 19 14:01:40 2021
@@ -17,49 +17,88 @@
 
 package org.apache.poi.ss.formula.functions;
 
-import static org.junit.jupiter.api.Assertions.assertEquals;
-
+import org.apache.poi.hssf.usermodel.HSSFCell;
+import org.apache.poi.hssf.usermodel.HSSFFormulaEvaluator;
+import org.apache.poi.hssf.usermodel.HSSFSheet;
+import org.apache.poi.hssf.usermodel.HSSFWorkbook;
+import org.apache.poi.ss.formula.OperationEvaluationContext;
 import org.apache.poi.ss.formula.eval.ErrorEval;
 import org.apache.poi.ss.formula.eval.NumberEval;
 import org.apache.poi.ss.formula.eval.StringEval;
 import org.apache.poi.ss.formula.eval.ValueEval;
 import org.junit.jupiter.api.Test;
 
+import java.io.IOException;
+
+import static org.apache.poi.ss.util.Utils.addRow;
+import static org.apache.poi.ss.util.Utils.assertDouble;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+
 /**
- * Tests for {@link org.apache.poi.ss.formula.functions.Delta}
+ * Tests for {@link TDist}
  */
-final class TestDelta {
+final class TestTDist {
+
+    private static final OperationEvaluationContext ec = new OperationEvaluationContext(null, null, 0, 0, 0, null);
 
-    private static ValueEval invokeValue(String number1, String number2) {
-        ValueEval[] args = new ValueEval[] { new StringEval(number1), new StringEval(number2), };
-        return new Delta().evaluate(args, -1, -1);
+    @Test
+    void testBasic() {
+        confirmValue("5.968191467", "8", "1", 0.00016754180265310392);
+        confirmValue("5.968191467", "8", "2", 0.00033508360530620784);
+        confirmValue("5.968191467", "8.2", "2.2", 0.00033508360530620784);
+        confirmValue("5.968191467", "8.9", "2.9", 0.00033508360530620784);
+    }
+
+    @Test
+    void testInvalid() {
+        confirmInvalidError("A1","B2","C2");
+        confirmInvalidError("5.968191467","8","C2");
+        confirmInvalidError("5.968191467","B2","2");
+        confirmInvalidError("A1","8","2");
     }
 
-    private static void confirmValue(String number1, String number2, double expected) {
-        ValueEval result = invokeValue(number1, number2);
+    @Test
+    void testNumError() {
+        confirmNumError("5.968191467", "8", "0");
+        confirmNumError("-5.968191467", "8", "2");
+    }
+
+    //https://support.microsoft.com/en-us/office/tdist-function-630a7695-4021-4853-9468-4a1f9dcdd192
+    @Test
+    void testMicrosoftExample1() throws IOException {
+        try (HSSFWorkbook wb = new HSSFWorkbook()) {
+            HSSFSheet sheet = wb.createSheet();
+            addRow(sheet, 0, "Data", "Description");
+            addRow(sheet, 1, 1.959999998, "Value at which to evaluate the distribution");
+            addRow(sheet, 2, 60, "Degrees of freedom");
+            HSSFFormulaEvaluator fe = new HSSFFormulaEvaluator(wb);
+            HSSFCell cell = wb.getSheetAt(0).getRow(0).createCell(100);
+            assertDouble(fe, cell, "TDIST(A2,A3,2)", 0.054644930, 0.01);
+            assertDouble(fe, cell, "TDIST(A2,A3,1)", 0.027322465, 0.01);
+        }
+    }
+
+    private static ValueEval invokeValue(String number1, String number2, String number3) {
+        ValueEval[] args = new ValueEval[] { new StringEval(number1), new StringEval(number2), new StringEval(number3)};
+        return TDist.instance.evaluate(args, ec);
+    }
+
+    private static void confirmValue(String number1, String number2, String number3, double expected) {
+        ValueEval result = invokeValue(number1, number2, number3);
         assertEquals(NumberEval.class, result.getClass());
         assertEquals(expected, ((NumberEval) result).getNumberValue(), 0.0);
     }
 
-    private static void confirmValueError(String number1, String number2) {
-        ValueEval result = invokeValue(number1, number2);
+    private static void confirmInvalidError(String number1, String number2, String number3) {
+        ValueEval result = invokeValue(number1, number2, number3);
         assertEquals(ErrorEval.class, result.getClass());
         assertEquals(ErrorEval.VALUE_INVALID, result);
     }
 
-    @Test
-    void testBasic() {
-        confirmValue("5","4", 0); // Checks whether 5 equals 4 (0)
-        confirmValue("5","5", 1); // Checks whether 5 equals 5 (1)
-
-        confirmValue("0.5","0", 0); // Checks whether 0.5 equals 0 (0)
-        confirmValue("0.50","0.5", 1);
-        confirmValue("0.5000000000","0.5", 1);
+    private static void confirmNumError(String number1, String number2, String number3) {
+        ValueEval result = invokeValue(number1, number2, number3);
+        assertEquals(ErrorEval.class, result.getClass());
+        assertEquals(ErrorEval.NUM_ERROR, result);
     }
 
-    @Test
-    void testErrors() {
-        confirmValueError("A1","B2");
-        confirmValueError("AAAA","BBBB");
-    }
 }



---------------------------------------------------------------------
To unsubscribe, e-mail: commits-unsubscribe@poi.apache.org
For additional commands, e-mail: commits-help@poi.apache.org
--- poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/functions/Delta.java (original)
+++ poi/trunk/poi/src/main/java/org/apache/poi/ss/formula/functions/TDist.java Tue Oct 19 14:01:40 2021
@@ -17,47 +17,79 @@
 
 package org.apache.poi.ss.formula.functions;
 
+import org.apache.commons.math3.distribution.TDistribution;
 import org.apache.poi.ss.formula.OperationEvaluationContext;
-import org.apache.poi.ss.formula.eval.ErrorEval;
-import org.apache.poi.ss.formula.eval.EvaluationException;
-import org.apache.poi.ss.formula.eval.NumberEval;
-import org.apache.poi.ss.formula.eval.OperandResolver;
-import org.apache.poi.ss.formula.eval.ValueEval;
+import org.apache.poi.ss.formula.eval.*;
 
 /**
  * Implementation for Excel DELTA() function.
  * <p>
- * <b>Syntax</b>:<br> <b>DELTA </b>(<b>number1</b>,<b>number2</b> )<br>
+ * <b>Syntax</b>:<br> <b>TDIST </b>(<b>X</b>,<b>Deg_freedom</b>,<b>Tails</b>)<br>
  * <p>
- * Tests whether two values are equal. Returns 1 if number1 = number2; returns 0 otherwise.
- * Use this function to filter a set of values. For example, by summing several DELTA functions
- * you calculate the count of equal pairs. This function is also known as the Kronecker Delta function.
+ * Returns the Percentage Points (probability) for the Student t-distribution where a numeric value (x)
+ * is a calculated value of t for which the Percentage Points are to be computed. The t-distribution is used
+ * in the hypothesis testing of small sample data sets. Use this function in place of a table of critical values
+ * for the t-distribution.
  *
  * <ul>
- *     <li>If number1 is nonnumeric, DELTA returns the #VALUE! error value.</li>
- *     <li>If number2 is nonnumeric, DELTA returns the #VALUE! error value.</li>
+ *     <li>X     Required. The numeric value at which to evaluate the distribution.</li>
+ *     <li>Deg_freedom     Required. An integer indicating the number of degrees of freedom.</li>
+ *     <li>Tails     Required. Specifies the number of distribution tails to return. If Tails = 1, TDIST returns the
+ *     one-tailed distribution. If Tails = 2, TDIST returns the two-tailed distribution.</li>
+ * </ul>
+ *
+ * <ul>
+ *     <li>If any argument is non-numeric, TDIST returns the #VALUE! error value.</li>
+ *     <li>If Deg_freedom < 1, TDIST returns the #NUM! error value.</li>
+ *     <li>The Deg_freedom and Tails arguments are truncated to integers.
+ *     <li>If Tails is any value other than 1 or 2, TDIST returns the #NUM! error value.</li>
+ *     <li>If x < 0, then TDIST returns the #NUM! error value.</li>
  * </ul>
  */
-public final class Delta extends Fixed2ArgFunction implements FreeRefFunction {
+public final class TDist extends Fixed3ArgFunction implements FreeRefFunction {
 
-    public static final FreeRefFunction instance = new Delta();
+    public static final TDist instance = new TDist();
 
-    private static final NumberEval ONE = new NumberEval(1);
-    private static final NumberEval ZERO = new NumberEval(0);
+    static double tdistOneTail(double x, int degreesOfFreedom) {
+        TDistribution tdist = new TDistribution(degreesOfFreedom);
+        return 1.0 - tdist.cumulativeProbability(x);
+    }
+
+    static double tdistTwoTails(double x, int degreesOfFreedom) {
+        return 2 * tdistOneTail(x, degreesOfFreedom);
+    }
 
     @Override
-    public ValueEval evaluate(int srcRowIndex, int srcColumnIndex, ValueEval arg1, ValueEval arg2) {
+    public ValueEval evaluate(int srcRowIndex, int srcColumnIndex, ValueEval arg1, ValueEval arg2, ValueEval arg3) {
         try {
             Double number1 = evaluateValue(arg1, srcRowIndex, srcColumnIndex);
             if (number1 == null) {
                 return ErrorEval.VALUE_INVALID;
+            } else if (number1 < 0) {
+                return ErrorEval.NUM_ERROR;
             }
             Double number2 = evaluateValue(arg2, srcRowIndex, srcColumnIndex);
             if (number2 == null) {
                 return ErrorEval.VALUE_INVALID;
             }
+            int degreesOfFreedom = number2.intValue();
+            if (degreesOfFreedom < 1) {
+                return ErrorEval.NUM_ERROR;
+            }
+            Double number3 = evaluateValue(arg3, srcRowIndex, srcColumnIndex);
+            if (number3 == null) {
+                return ErrorEval.VALUE_INVALID;
+            }
+            int tails = number3.intValue();
+            if (!(tails == 1 || tails == 2)) {
+                return ErrorEval.NUM_ERROR;
+            }
+
+            if (tails == 2) {
+                return new NumberEval(tdistTwoTails(number1, degreesOfFreedom));
+            }
 
-            return (number1.compareTo(number2) == 0) ? ONE : ZERO;
+            return new NumberEval(tdistOneTail(number1, degreesOfFreedom));
         } catch (EvaluationException e) {
             return e.getErrorEval();
         }
@@ -65,8 +97,8 @@ public final class Delta extends Fixed2A
 
     @Override
     public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
-         if (args.length == 2) {
-            return evaluate(ec.getRowIndex(), ec.getColumnIndex(), args[0], args[1]);
+         if (args.length == 3) {
+            return evaluate(ec.getRowIndex(), ec.getColumnIndex(), args[0], args[1], args[2]);
         }
 
         return ErrorEval.VALUE_INVALID;
@@ -17,47 +17,79 @@
 
 /**
- * Implementation for Excel DELTA() function.
+ * Implementation for Excel TDIST() function.
  * <p>

Copied: poi/trunk/poi/src/test/java/org/apache/poi/ss/formula/functions/TestTDist.java (from r1894356, poi/trunk/poi/src/test/java/org/apache/poi/ss/formula/functions/TestDelta.java)
URL: http://svn.apache.org/viewvc/poi/trunk/poi/src/test/java/org/apache/poi/ss/formula/functions/TestTDist.java?p2=poi/trunk/poi/src/test/java/org/apache/poi/ss/formula/functions/TestTDist.java&p1=poi/trunk/poi/src/test/java/org/apache/poi/ss/formula/functions/TestDelta.java&r1=1894356&r2=1894378&rev=1894378&view=diff
==============================================================================